import os
import csv
import openai
import requests
import time
import json
import re
import winsound
import sys
import pandas as pd
import traceback
from tqdm import tqdm
from dotenv import load_dotenv
from datetime import datetime
from account_summary import get_account_summary_df

load_dotenv()
webhook_url = os.getenv("webhook_url")

def print_with_sound(message):
	print(message)
	winsound.PlaySound("path/to/sound.wav", winsound.SND_FILENAME)

class chatTime:
	def __init__(self):
		self.openai_api_key = os.getenv("openai_api_key")
		openai.api_key = self.openai_api_key
		self.timestamp = None 
	
	def get_timestamp(self):
		current_time = datetime.now()
		timestamp = current_time.strftime("%Y%m%d_%H%M%S")
		return timestamp
	
	def get_data(self):
		try:
			with open('ta_data.csv', 'r') as file:
				reader = csv.reader(file)
				data = list(reader)
				headers = data[0]
				last_30_rows = data[-31:]
				newest_row = data[-1:]
				return headers, last_30_rows, newest_row
		except Exception as e:
			print("An error occurred while trying to read the CSV file:", str(e))

	def chat_with_gpt(self, message):
		try:
			response = openai.ChatCompletion.create(
				model="gpt-4",
				messages=[
					{"role": "system", "content": "You are a seasoned cryptocurrency trader in the fast-paced world of financial markets, known for your analytical prowess."},
					{"role": "user", "content": message}
				]
			)
			response_text = response.choices[0].message['content']
			
			self.save_to_file("Response:", response_text)

			return response_text

		except Exception as e:
			print("An error occurred while trying to chat with GPT:", str(e))
			print(str(e))
			return str(e)

	def save_to_file(self, message_type, message_content):
		current_time = datetime.now()
		date_str = current_time.strftime("%Y%m%d")
		hour_str = current_time.strftime("%H")

		prompts_dir = "prompts"
		if not os.path.exists(prompts_dir):
			os.makedirs(prompts_dir)

		date_dir = os.path.join(prompts_dir, date_str)
		if not os.path.exists(date_dir):
			os.makedirs(date_dir)
		hour_dir = os.path.join(date_dir, hour_str)
		if not os.path.exists(hour_dir):
			os.makedirs(hour_dir)

		if self.timestamp is None:
			self.timestamp = self.get_timestamp()

		initial_file = os.path.join(hour_dir, f"{self.timestamp}.txt")

		if message_type == "Prompt: " and not os.path.exists(initial_file):
			with open(initial_file, "a") as f:
				f.write(f"{message_type}\n{message_content}\n")
		else:
			with open(initial_file, "a") as f:
				f.write(f"{message_type}\n{message_content}\n")

def main():
	try: 
		chat_time = chatTime()
		feedback = None
		
		while True:
			headers, last_30_rows, newest_row = chat_time.get_data()
			
			try:
				df_account = get_account_summary_df()
				account_summary_str = df_account.to_string(index=False)
			except Exception as e:
				account_summary_str = f"Unable to fetch account summary due to: {str(e)}"
				
			if feedback:
				prompt = f"""
You're being given a CSV file containing real-time candlestick data for ethusdt futures, obtained every 5 minutes from crypto.com. The data for each row includes:

{headers}

Here are a few data entries for your reference:

{last_30_rows}

Your mission is to scrutinize the data and use scalping techniques to identify short-term price inefficiencies and exploit them for quick profits by signaling a buy alert. Employ the provided technical indicators, to identify potential entry and exit points.You have one main goal, increase our net worth.
The last trade signal you provided received this feedback: {feedback}

Your current account summary is :
{account_summary_str}

It is also important to consider a 0.075% trading fee per order, this is a total of 1.5% to open and close a trade. Consider this when choosing your exits.
			
This system is automated and therefore must focus on minimizing characters. Consequently, you should directly provide the decision derived from your analysis by providing your comments on the feedback and how it affected your latest decision AND your selection from the following options of Autoview commands at the end of your response while replacing the __ in the options with actual stop loss [fpx=] and take profit [fp=] values.

- s=eth_usd b=buy q=1% p=5% d=1|s=eth_usd b=sell fpx=__ t=market d=1|s=eth_usd b=sell fp=__ d=1
- Do Nothing

Here's an example of how your response should be structured for your guidance. :
s=eth_usd b=buy q=1% p=5% d=1|s=eth_usd b=sell fpx=1880 t=market d=1|s=eth_usd b=sell fp=1910 d=1
"""
			else:
				prompt = f"""
You're being given a CSV file containing real-time candlestick data for ethusdt futures, obtained every 5 minutes from crypto.com. The data for each row includes:

{headers}

Here are a few data entries for your reference:

{last_30_rows}

Your mission is to scrutinize the data and use scalping techniques to identify short-term price inefficiencies and exploit them for quick profits by signaling a buy alert. Employ the provided technical indicators, to identify potential entry and exit points.You have one main goal, increase our net worth.

Your current account summary is :
{account_summary_str}

It is also important to consider a 0.075% trading fee per order, this is a total of 1.5% to open and close a trade. Consider this when choosing your exits.

This system is automated and therefore must focus on minimizing characters. Consequently, you should directly provide the decision derived from your analysis by providing your selection from the following options of Autoview commands at the end of your response while replacing the __ in the options with actual stop loss [fpx=] and take profit [fp=] values.

- s=eth_usd b=buy q=1% p=1% d=1|s=eth_usd b=sell fpx=__ t=market d=1|s=eth_usd b=sell fp=__ d=1
- Do Nothing

Here's an example of how your response should be structured for your guidance. :
s=eth_usd b=buy q=1% p=1% d=1|s=eth_usd b=sell fpx=1880 t=market d=1|s=eth_usd b=sell fp=1910 d=1
"""

			while True:
				chat_time.save_to_file("Prompt: ", prompt)
				print("Talking to chatGPT")	
				print("~~~~~~~~~~~~~~~~~~~~~~~~")
				response = chat_time.chat_with_gpt(prompt)
#				print(prompt)
#				print("~~~~~~~~~~~~~~~~~~~~~~~~")
				print(response) 
				print("~~~~~~~~~~~~~~~~~~~~~~~~")
				
				if "An error occurred while trying to chat with GPT:" in response:
					print("Skipping this iteration due to error.")
					for remaining_time in tqdm(range(20), desc="~"):
						time.sleep(1)
					continue
					
				# Check response and take action
				if "b=buy" in response:  # buy or sell signal
					
					try:
						autoview_command_pattern = r"s=eth_usd b=[a-z]+ q=[0-9%]+ p=[0-9%]+|s=eth_usd b=sell fpx=[0-9,.]+ t=market|s=eth_usd b=sell fp=[0-9,.]+"
						matches = re.findall(autoview_command_pattern, response)

						if matches:
							autoview_command = " | ".join(matches)
							# You can use 'autoview_command' as the message in your webhook
							r = requests.post(webhook_url, data=autoview_command.encode('utf-8'))
						else:
							print("No Autoview command found in the response.")
							r = None  # or any other default value
						
						# If the request was successful, r.status_code will be 200
						if r.status_code == 200:
							# print(prompt)
							print("Successfully sent to Autoview.com:" , autoview_command)
							feedback = input("Please provide feedback on the closed trade: ")
							break
						else:
							print(f"Failed to send. Status code: {r.status_code}")
					except Exception as e:
						print(f"An error occurred: {e}")

				else:
					# sys.exit()
					for remaining_time in tqdm(range(60*5), desc="~"):
						time.sleep(1)
					break
					
	except Exception as e:
		print("An error occurred while trying to read the CSV file:", str(e))
		traceback.print_exc()
if __name__ == "__main__":
	main()